@@ -1,5 +1,3 @@
-use std::cmp::Ordering;
-use std::cmp::Ordering::{Greater, Less};
 use std::fmt::{Debug, Display};
 use std::ops::{Add, BitAnd, BitOrAssign, Shl, Shr, Sub};
 
@@ -44,16 +42,21 @@ pub trait SignedLike {
 /// Under the hood, when numbers are encoded or decoded, they go through their
 /// corresponding `UnsignedLike` representation.
 pub trait UnsignedLike: Add<Output=Self> + BitAnd<Output=Self> + BitOrAssign +
-Copy + Debug + Display + From<u8> + Ord + PartialOrd +
+Copy + Debug + Display + Ord + PartialOrd +
 Shl<usize, Output=Self> + Shr<usize, Output=Self> + Sub<Output=Self> {
   const ZERO: Self;
   const ONE: Self;
   const MAX: Self;
   const BITS: usize;
 
+  /// Converts a `usize` into this type. Panics if the conversion is
+  /// impossible.
+  fn from_word(word: usize) -> Self;
+
   fn to_f64(self) -> f64;
 
-  /// Shifts the unsigned integer right and returns its lowest bits as a usize.
+  /// Shifts the unsigned integer right and returns its lowest bits as a
+  /// `usize`.
   /// For example,
   /// ```
   /// use q_compress::data_types::UnsignedLike;
@@ -64,7 +67,8 @@ Shl<usize, Output=Self> + Shr<usize, Output=Self> + Sub<Output=Self> {
   /// Used for some bit arithmetic operations during compression.
   fn rshift_word(self, shift: usize) -> usize;
 
-  /// Shifts the unsigned integer left and returns its lowest bits as a usize.
+  /// Shifts the unsigned integer left and returns its lowest bits as a
+  /// `usize`.
   /// For example,
   /// ```
   /// use q_compress::data_types::UnsignedLike;
@@ -95,14 +99,14 @@ Shl<usize, Output=Self> + Shr<usize, Output=Self> + Sub<Output=Self> {
 pub trait NumberLike: Copy + Debug + Display + Default + PartialEq + 'static {
   /// A number from 0-255 that corresponds to the number's data type.
   ///
-  /// Each `NumberLike` implementation should have a different `HeaderByte`.
+  /// Each `NumberLike` implementation should have a different `HEADER_BYTE`.
   /// This byte gets written into the file's header during compression, and
   /// if the wrong header byte shows up during decompression, the decompressor
   /// will return an error.
   ///
   /// To choose a header byte for a new data type, review all header bytes in
   /// the library and pick an unused one. For instance, as of writing, bytes
-  /// 1 through 10 are used, so 11 would be a good choice for another
+  /// 1 through 13 are used, so 14 would be a good choice for another
   /// `q_compress`-supported data type, and 255 would be a good choice for a
   /// custom data type.
   const HEADER_BYTE: u8;
@@ -113,27 +117,22 @@ pub trait NumberLike: Copy + Debug + Display + Default + PartialEq + 'static {
   /// and timestamps have 96 (not 128).
   const PHYSICAL_BITS: usize;
 
-  /// The unsigned integer this type can convert between to do
-  /// bitwise logic and such.
-  type Unsigned: UnsignedLike;
   /// The signed integer this type can convert between to do wrapped
   /// subtraction and addition for delta encoding/decoding.
   /// Must be another `NumberLike` with the same `Signed` and `Unsigned` as
   /// this type; in this way, if we take 7th order deltas, they are ensured to
   /// have the same type as 1st order deltas.
   type Signed: SignedLike + NumberLike<Signed=Self::Signed, Unsigned=Self::Unsigned>;
+  /// The unsigned integer this type can convert between to do
+  /// bitwise logic and such.
+  type Unsigned: UnsignedLike;
 
   /// Lossless check for bit-exact equality. This is important because not all data types
   /// support full ordering:
   /// <https://stackoverflow.com/questions/26489701/why-does-rust-not-implement-total-ordering-via-the-ord-trait-for-f64-and-f32>.
-  fn num_eq(&self, other: &Self) -> bool;
-
-  /// Lossless numerical comparison. This is important for the same reason as
-  /// `num_eq`.
-  /// We use it to sort numbers and calculate quantiles.
-  /// For example, this function can order the many `f32` and `f64` NaN
-  /// representations.
-  fn num_cmp(&self, other: &Self) -> Ordering;
+  fn num_eq(&self, other: &Self) -> bool {
+    self.to_unsigned() == other.to_unsigned()
+  }
 
   /// Used during compression to convert to an unsigned integer.
   fn to_unsigned(self) -> Self::Unsigned;
@@ -165,20 +164,4 @@ pub trait NumberLike: Copy + Debug + Display + Default + PartialEq + 'static {
   fn write_to(self, writer: &mut BitWriter) {
     writer.write(&bits::bytes_to_bits(self.to_bytes()));
   }
-
-  fn le(&self, other: &Self) -> bool {
-    !matches!(self.num_cmp(other), Greater)
-  }
-
-  fn lt(&self, other: &Self) -> bool {
-    matches!(self.num_cmp(other), Less)
-  }
-
-  fn ge(&self, other: &Self) -> bool {
-    !matches!(self.num_cmp(other), Less)
-  }
-
-  fn gt(&self, other: &Self) -> bool {
-    matches!(self.num_cmp(other), Greater)
-  }
 }
