@@ -1,4 +1,3 @@
-use std::cmp::Ordering;
 use std::convert::TryInto;
 
 use crate::data_types::{NumberLike, SignedLike};
@@ -41,14 +40,6 @@ macro_rules! impl_signed {
         Self::MIN.wrapping_add(off as $t)
       }
 
-      fn num_eq(&self, other: &Self) -> bool {
-        self.eq(other)
-      }
-
-      fn num_cmp(&self, other: &Self) -> Ordering {
-        self.cmp(other)
-      }
-
       fn to_bytes(self) -> Vec<u8> {
         self.to_be_bytes().to_vec()
       }
@@ -60,6 +51,7 @@ macro_rules! impl_signed {
   }
 }
 
+impl_signed!(i16, u16, 13);
 impl_signed!(i32, u32, 3);
 impl_signed!(i64, u64, 1);
 impl_signed!(i128, u128, 10);
