@@ -1,4 +1,3 @@
-use std::cmp::Ordering;
 use std::convert::TryInto;
 
 use crate::data_types::{NumberLike, UnsignedLike};
@@ -12,6 +11,10 @@ macro_rules! impl_unsigned {
       const MAX: Self = Self::MAX;
       const BITS: usize = Self::BITS as usize;
 
+      fn from_word(word: usize) -> Self {
+        word as Self
+      }
+
       fn to_f64(self) -> f64 {
         self as f64
       }
@@ -38,6 +41,7 @@ macro_rules! impl_unsigned {
 }
 
 impl_unsigned!(u8);
+impl_unsigned!(u16);
 impl_unsigned!(u32);
 impl_unsigned!(u64);
 impl_unsigned!(u128);
@@ -67,14 +71,6 @@ macro_rules! impl_unsigned_number {
         off
       }
 
-      fn num_eq(&self, other: &Self) -> bool {
-        self.eq(other)
-      }
-
-      fn num_cmp(&self, other: &Self) -> Ordering {
-        self.cmp(other)
-      }
-
       fn to_bytes(self) -> Vec<u8> {
         self.to_be_bytes().to_vec()
       }
@@ -86,5 +82,7 @@ macro_rules! impl_unsigned_number {
   }
 }
 
+impl_unsigned_number!(u16, i16, 12);
 impl_unsigned_number!(u32, i32, 4);
 impl_unsigned_number!(u64, i64, 2);
+impl_unsigned_number!(u128, i128, 11);
