@@ -42,4 +42,9 @@ fn test_v04_f32() {
 #[test]
 fn test_v06_timestamp_deltas() {
   assert_compatible::<TimestampMicros>("v0.6_timestamp_deltas_2k");
-}
\ No newline at end of file
+}
+
+#[test]
+fn test_v09_dispersed_shorts() {
+  assert_compatible::<u16>("v0.9_dispersed_shorts");
+}
